/** This is the header for the auxiliary functions of program CUBETRANSPORT **/

#ifndef AUXFUNCTIONS_H_INCLUDED
#define AUXFUNCTIONS_H_INCLUDED


/** MACROS FOR CONSTANTS **/

#define CONST_0_618 .61803398874989485 // (-1+sqrt(5))/2
#define CONST_0_666 .66666666666666667 // 2/3
#define CONST_0_999 .99999000000000000 // 1-10**(-5)
#define CONST_1_618 1.6180339887498949 // (sqrt(5)+1)/2
#define CONST_1_959 1.9599639845400538 // sqrt(2)*erfinv(0.95)
#define CONST_2_718 2.7182818284590452 // e
#define CONST_2_828 2.8284271247461903 // sqrt(8)
#define CONST_2_833 2.8333333333333333 // 2+5/6
#define CONST_8_315 8.3152410503158567 // 2**(4+3/4)/(1+sqrt(5))
#define CONST_8_500 8.5000000000000000 // 8+1/2


/** FUNCTIONAL MACROS **/

/* Macro "METAMALLOC" allocates memory for a meta-array "array" (of any type of the form (type ***)) of size "nnn" (of type (struct int3)) */

#define METAMALLOC(array,nnn) array = malloc (nnn.x * sizeof(*array)); for (int x = 0; x < nnn.x; ++x) {array[x] = malloc (nnn.y * sizeof(**array)); for (int y = 0; y < nnn.y; ++y) {array[x][y] = malloc (nnn.z * sizeof(***array));}}

/* Macro "METAFREE" frees a meta-array "array" (of any type of the form (type ***)) of size "nnn" (of type (struct int3)) */

#define METAFREE(nnn,array) for (int x = 0; x < nnn.x; ++x) {for (int y = 0; y < nnn.y; ++y) {free (array[x][y]);} free (array[x]);} free (array)

/* Macro "METAFOR" makes variables "x", "y" and "z" (of type int) run along all triples of integers less than "nnn" (of type (struct int3)) */

#define METAFOR(x,y,z,nnn) for (x = 0; x < nnn.x; ++x) {for (y = 0; y < nnn.y; ++y) {for (z = 0; z < nnn.z; ++z)

/* Macro "ENDMETAFOR" is used to end a METAFOR loop */

#define ENDMETAFOR }}


/** FUNCTION PROTOTYPES **/

/* Function "det" computes the determinant of a 3*3 matrix "m" */

double det (struct double33 m);

/* Function "dist" computes the Euclidian distance between points "p0" and "p1" */

double dist (struct double3 p0, struct double3 p1);

/* Function "error" emits an error message on both "stderr" and "logstream", that message given by "format" and the extra arguments; then the program exits with status EXIT_FAILURE */

void error (char const *restrict format, ...);

/* Function "ffprintf" makes the same work as the standard "fprintf", except that it prints its message onto several streams, whose number is indicated by "z" */

void ffprintf (int z, ...);

/* Function "flowat" reads the value of the flow array "flow" at the index (x,y,z,v); yet if that index does not correspond to a correct flow, it returns 0 */

double flowat (double (***flow)[3], int x, int y, int z, int v);

/* Function "fopen_ctrl" makes the same work as the standard "fopen", except that in case of failure it emits an error and exits the program */

FILE *fopen_ctrl (char const *restrict filename, char const *restrict mode);

/* Function "isround" selects exponentially spaced integers: "isround (i)" is true each time log(i) passes a half-power of phi */

bool isround (int i);

/* Function "lincomb" computes the vector whose coordinates in basis "lattice" are (ax,ay,az) */

struct double3 lincomb (double ax, double ay, double az);

/* Function "okflow" tells whether the quadruple (x,y,z,v) really corresponds to a flow */

bool okflow (int x, int y, int z, enum dirflow v);

/* Function "okloop" tells whether the quadruple (x,y,z,w) really corresponds to a loop */

bool okloop (int x, int y, int z, enum dirloop w);

/* Function "rand1" draws a random number uniformly in (0,1) */

double rand1 (void);

/* Function "scalprod" computes the scalar product between two vectors "u" and "v" */

double scalprod (struct double3 u, struct double3 v);

/* Function "strncpy_0" makes the same work as the standard "strncpy", except that it ensures that the copied string is null-terminated */

void strncpy_0 (char *restrict dest, const char *restrict src, size_t count);

/* Function "vectmult" computes a.v for a vector "v" and a scalar "a" */

struct double3 vectmult (double a, struct double3 v);

/* Function "vectsum" computes v+w for two vectors "v" and "w" */

struct double3 vectsum (struct double3 v, struct double3 w);

/* Function "warning" emits a warning message on both "stderr" and "logstream", that message given by "format" and the extra arguments; then processing continues */

void warning (char const *restrict format, ...);

#endif // AUXFUNCTIONS_H_INCLUDED
