#include "cubetransport.h"
#include "quadratic.h"

void initflow (double ***charge0, double ***charge1, double (***flow)[3])
{


  /** VARIABLE DECLARATIONS **/

  double ***chargediff; // Array of the charge differences
  double ***charges; // Array of the sums of charges
  int x, y, z; // Indices for locating a node in the grid
  double tomove, tottomove; // Charge to move between two adjacent cells, resp. total charge to move for a given step
  double ref, totref; // Coefficient to weigth the charge moves, resp. total of the coefficients for a given step


  /** PRELIMINARY COMPUTATIONS **/

  METAMALLOC (chargediff, nnn);
  METAMALLOC (charges, nnn);
  METAFOR (x, y, z, nnn)
  {
    chargediff[x][y][z] = charge0[x][y][z] - charge1[x][y][z];
    charges[x][y][z] = charge0[x][y][z] + charge1[x][y][z];
  }
  ENDMETAFOR


  /** FIRST STEP: BALANCING BETWEEN THE ISO-x PLANES **/

  for (x = 0; x < nnn.x - 1; ++x)
    {
      tottomove = 0;
      totref = 0;
      for (y = 0; y < nnn.y; ++y)
        {
          for (z = 0; z < nnn.z ; ++z)
            {
              tottomove += chargediff[x][y][z];
              ref = charges[x][y][z] + charges[x + 1][y][z];
              totref += ref;
            }
        }
      for (y = 0; y < nnn.y; ++y)
        {
          for (z = 0; z < nnn.z ; ++z)
            {
              ref = charges[x][y][z] + charges[x + 1][y][z];
              tomove = ref / totref * tottomove;
              flow[x][y][z][X] = tomove;
              chargediff[x][y][z] -= tomove;
              chargediff[x + 1][y][z] += tomove;
            }
        }
    }


  /** SECOND STEP: BALANCING BETWEEN THE ISO-y LINES **/

  for (x = 0; x < nnn.x; ++x)
    {
      for (y = 0; y < nnn.y - 1; ++y)
        {
          tottomove = 0;
          totref = 0;
          for (z = 0; z < nnn.z; ++z)
            {
              tottomove += chargediff[x][y][z];
              ref = charges[x][y][z] + charges[x][y + 1][z];
              totref += ref;
            }
          for (z = 0; z < nnn.z; ++z)
            {
              ref = charges[x][y][z] + charges[x][y + 1][z];
              tomove = ref / totref * tottomove;
              flow[x][y][z][Y] = tomove;
              chargediff[x][y][z] -= tomove;
              chargediff[x][y + 1][z] += tomove;
            }
        }
    }


  /** THIRD STEP: BALANCING BETWEEN THE z POSITIONS **/

  for (x = 0; x < nnn.x; ++x)
    {
      for (y = 0; y < nnn.y; ++y)
        {
          for (z = 0; z < nnn.z - 1; ++z)
            {
              tomove = chargediff[x][y][z];
              flow[x][y][z][Z] = tomove;
              chargediff[x][y][z] = 0;
              chargediff[x][y][z + 1] += tomove;
            }
        }
    }


  /** EXITING THE FUNCTION **/

  METAFREE (nnn, charges);
  METAFREE (nnn, chargediff);
  return;
}
