#include "cubetransport.h"
#include "quadratic.h"

double lqdistance (double ***charge0, double ***charge1)
{


  /** VARIABLE DECLARATIONS **/

  double (***flow)[3]; // Transportation plan, expressed as a flow structure
  double cost; // Transportation cost
  double distance, lastvalues[3]; // Transportation distance (square root of cost), and last distances computed
  double zero_descent; // Threshold to halt the gradient descent
  int iteration; // Iteration number


  /** PREPARING STUFF FOR THE GRADIENT DESCENT **/

  printf ("Initializing the transportation plan to optimize...\n");
  METAMALLOC (flow, nnn);
  initflow (charge0, charge1, flow);
  lastvalues[2] = HUGE_VAL;
  lastvalues[1] = HUGE_VAL;
  lastvalues[0] = HUGE_VAL;
  zero_descent = ZERO_DESCENT_DEFAULT / pow (2, precision);
  fprintf (logstream, "The threshold to consider stationarity as attained has been set at %.2E a.u..\n", zero_descent);


  /** GRADIENT DESCENT **/

  ffprintf (2, stdout, logstream, "Iterating for the minimal transportation cost...\n");
  for (iteration = 0; true; ++iteration)
    {

      /* We compute the transportation distance at every "round" iteration */

      if (isround (iteration))
        {
          cost = qcost (flow, charge0, charge1);
          distance = sqrt (cost);
          ffprintf (2, stdout, logstream, "Iteration #%d: %f a.u..\n", iteration, distance);
          fflush (logstream);

          /* Checking if stationarity has been attained */

          lastvalues[2] = lastvalues[1];
          lastvalues[1] = lastvalues[0];
          lastvalues[0] = distance;
          if (lastvalues[2] - lastvalues[0] < zero_descent && iteration >= ITERATION_MIN)
            {
              ffprintf (2, stdout, logstream, "Stationarity has been attained!\n");
              break;
            }
        }

      /* Descent step */

      descent (flow, charge0, charge1);
    }


  /** EXITING THE FUNCTION **/

  METAFREE (nnn, flow);
  return distance;
}
