/** This is the header for the functions of program CUBETRANSPORT that are used to compute (quadratic Wasserstein)-like transportation distances [modes "qw" and "lq"] by following a grandient descent on some charge flow **/

#ifndef QUADRATIC_H_INCLUDED
#define QUADRATIC_H_INCLUDED


/** REQUIRED HEADERS **/

#include "flow.h"


/** GLOBAL VARIABLES **/

int t1; // Number of discretization steps for time-continuous transportation


/** FUNCTION PROTOTYPES **/

/* Function "qcost" computes the (linearized) quadratic transportation cost of a flow "flow", the charge environment being the mean of "charge0" and "charge1" */

double qcost (double (***flow)[3], double ***charge0, double ***charge1);

/* Function "descent" takes a flow "flow" and applies to it a step of the gradient descent for the (linearized) quadratic transportation cost, the charge environment being the mean between "charge0" and "charge1" */

void descent (double (***flow)[3], double ***charge0, double ***charge1);

/* Function "codescent" performs a gradient descent step to optimize the total (linearized) quadratic cost along time for transporting a fixed total amount of charge between two neighbor nodes A and B: "flow" (which means the values pointed to by "p_flow") is the charge transported along time; "chargea", resp. "chargeb" (pointed to by resp. "p_chargea" and "p_chargeb"), are the charges at the start point A, resp. at the finish point B, along time; "dipolea", resp. "dipoleb", are the flow currents (expressed as dipoles) around point A, resp. around point B, along time; and "v" is the direction along which A and B are adjacent */

void codescent (double *p_flow[], double *p_chargea[], double *p_chargeb[], struct double3 dipolea[], struct double3 dipoleb[], enum dirflow v);

#endif // QUADRATIC_H_INCLUDED
